/**
 * Copyright 2012 Bull S.A.S.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.ow2.util.testng4osgi.launcher;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.osgi.framework.BundleContext;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.maven.osgi.launcher.test.api.ITestConfiguration;
import org.ow2.util.maven.osgi.launcher.test.api.ITestLauncher;
import org.ow2.util.maven.osgi.launcher.test.api.ITestReport;
import org.ow2.util.testng4osgi.factory.TestNGObjectFactory;
import org.ow2.util.testng4osgi.listener.TestNGTestListener;
import org.testng.TestNG;

/**
 * Allows to start the TestNG tests. 
 * @author Florent Benoit
 */
public class TestNGLauncher implements ITestLauncher {

    /**
     * Logger.
     */
    private static final Log LOGGER = LogFactory.getLog(TestNGLauncher.class);

    
    /**
     * List of TestNG objects.
     */
    private Map<BundleContext, EnhancedTestNG> testNGs = null;
    
    /**
     * Report listener.
     */
    private TestNGTestListener reportListener = null;
    
    /**
     * Default constructor.
     */
    public TestNGLauncher() {
        this.testNGs = new HashMap<BundleContext, EnhancedTestNG>();
        this.reportListener  = new TestNGTestListener();
    }
    
    /**
     * Adding new tests for the given bundle context.
     * @param bundleContext the bundle context of the given classes
     * @param classes the set of classes to add
     */
    public void addTests(BundleContext bundleContext, List<Class<?>> classes) {
        
        // new TestNG engine for each set of classes
        EnhancedTestNG testNG = new EnhancedTestNG(bundleContext);
        
        // define classes
        testNG.setOSGiClasses(classes);
        
        // Set our own factory in order to be able to manage dependency injection
        testNG.setObjectFactory(new TestNGObjectFactory(bundleContext));
     
        // Listener used to manage failures
        testNG.addListener(reportListener);
        
        //TODO: remove debug
        // Increase verbosity for now
        //testNG.setVerbose(10);
        
        // Add engine to the set
        testNGs.put(bundleContext, testNG);
        
    }
    
    /**
     * Run the given tests by using the given configuration.
     * @param configuration the test configuration
     * @return a report
     */
    public ITestReport runTests(ITestConfiguration configuration) {
        
        // Update configuration
        for (TestNG testNG : testNGs.values()) {
            if (configuration.getOutputDirectory() != null) {
                testNG.setOutputDirectory(configuration.getOutputDirectory().getAbsolutePath());
            }
        }

        // Run
        for (TestNG testNG : testNGs.values()) {
            testNG.run();
        }
        
        // Analyze results
        int status = 0;
        for (TestNG testNG : testNGs.values()) {
            status = status | testNG.getStatus();
        }
        
        LOGGER.info("Report : Run ''{0}'', Failures : ''{1}'', Skipped : ''{2}'', Success : ''{3}''",
                reportListener.getTestStart(), reportListener.getFailures() ,reportListener.getSkipped() ,
                reportListener.getTestStart() ,reportListener.getTestSuccess());
        
        
        return new TestReport(0 == status);
    }

}
